
 /* nxlcau2t.c				*/
/* (2,3/2) Linear Cellular Automaton	*/

/* Reference:				*/
/*					*/
/*    Kenneth E. Perry			*/
/*    Abstract Mathematical Art		*/
/*    BYTE				*/
/*    December, 1986			*/
/*    pages 181-192			*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1990		*/
/*    Copyright (C) 1994		*/
/*    Harold V. McIntosh		*/
/*    Gerardo Cisneros S.		*/

/* G. Cisneros, 4.3.87						*/
/* 28 April 1987 - version for XVI Feria de Puebla [HVM]	*/
/* 29 June 1987 - conserve position of cell pointer [HVM]	*/
/* 30 June 1987 - mark & unmark transitions, xchg x&X [HVM]	*/
/* 25 July 1987 - display and edit de Bruijn diagrams [HVM]	*/
/* 27 July 1987 - graph of transition probabilities [HVM]	*/
/* 4 September 1987 - PROB21.C for option 't' [HVM]		*/
/* 21 October 1987 - program disks disappeared			*/
/* 20 December 1987 - program reconstructed from listings	*/
/* 20 February 1988 - RIJN21.C for option 'd' [HVM]		*/
/* 11 March 1989 - x option in trule [HVM]			*/
/* 16 March 1989 - *+- option to change screen color [HVM]	*/
/* 17 September 1989 - directory for demonstration rules [HVM]	*/
/* 14 October 1989 - modified for r = 3/2 [HVM]			*/
/* 10 February 1990 - Garden of Eden option [HVM] 		*/

/* # include <bdos.h> */
# include "nxlcau2t.h"
# include "rec.h"
# include "keybrd.h"


# define LROW		 9	/* top row of line panel  */

char ixrule[NX][DS+1]= {
	"0001010101101110",	/*  1 close tangency      */
	"0001011001101001",	/*  2 totalistic rule #20 */
	"0001011001101000",	/*  3 life candidate 1    */
	"0001011001101010",	/*  4 life candidate 2    */
	"0001011011010110",	/*  5 coexistence         */
	"0001011101101010",	/*  6 life candidate 3	  */
	"0001011111101001",	/*  7 curdles             */
	"0001101110110101",	/*  8 zigzags             */
	"0010111101010110",	/*  9 fishnet             */
	"0011101000111100",	/* 10 zero variance       */
	"0011110010100101",	/* 11 class iii sm exp #1 */
	"0101001100001010",	/* 12 hanging kelp bed    */
	"0101101000100110",	/* 13 class iii sm exp #2 */
	"0110100110010111",	/* 14 totalistic rule #26 */
	"1011100000000011",	/* 15 floating triangles  */
	"1101001011110000"	/* 16 reversible rule  	  */
	};

char recrule[NY][CLEN]= {
	"(\"5486\",	W;[ 1 close tangency      ])",
	"(\"5737\",	W;[ 2 totalistic rule #20 ])",
	"(\"5736\",	W;[ 3 life candidate 1    ])",
	"(\"5738\",	W;[ 4 life candidate 2    ])",
	"(\"5846\",	W;[ 5 coexistence         ])",
	"(\"5994\",	W;[ 6 life candidate 3    ])",
	"(\"6121\",	W;[ 7 curdles             ])",
	"(\"1BB5\",	W;[ 8 zigzags             ])",
	"(\"2F56\",	W;[ 9 fishnet             ])",
	"(\"3A1E\",	W;[10 zero variance       ])",
	"(\"3CA5\",	W;[11 class iii sm exp #1 ])",
	"(\"530A\",	W;[12 hanging kelp bed    ])",
	"(\"5A26\",	W;[13 class iii sm exp #2 ])",
	"(\"69C7\",	W;[14 totalistic rule #26 ])",
	"(\"B803\",	W;[15 floating triangles  ])",
	"(\"D2F0\"	W;[16 reversible rule     ])" 
	};


parneih    amat;
parneih    bmat;
parneih    fmat;
parneih    gmat;
parneih    hmat;
parneih    xmat;

twoparn    smat;
twoparn    tmat;
twoparn    umat;
twoparn    vmat;

extern struct	fptbl dtbl[];
void  chrucopy(char ii[KK][KK][KK][KK],char jj[KK][KK][KK][KK],char c);
void   wolnum(int ii[KK][KK][KK][KK], int c);

char   strule[KK+1][TS-2];
char   gf2rule[13];			/* coefficients of GF2 rule	*/
char   ttrule[TS];
char   tarule[TS];
char   srule[KK+1][TS-2];
char   ascrule1[KK][KK][KK];
char   ascrule[KK][KK][KK][KK];
char   qqascrule[KK][KK][KK][KK][KK][KK][KK][KK][KK];
char   ascrulo[KK][KK][KK];
char   auxrule1[KK][KK][KK]; 
char   auxrule[KK][KK][KK][KK];		/* auxiliary transition table */
int    binrule[KK][KK][KK][KK];
extern int    cc, dd;			/* radius of de Bruijn ring */

extern int	rec_x(Inst rprg);

char   arr1[AL], arr2[AL], arr3[AL];
int    ru, ru0, ru1, ru2, ru3;			/* rule cursor */
int    co, dn;					/* connectivity, dimension */
int    pr, pl;					/* probability, in mils    */
int    ix0, iy0;				/* origin for pen moves    */
int    dij, di, dj;				/* demo pointer            */
int    rij, rdi, rdj;				/* rec demo pointers 	   */
int    li, lj;					/* cursor for edline       */
int    bi;					/* bernstein index	   */
int    rp, rq;					/* source code in cstr	   */
int    sw0, sw1, sw2, sw3;			/* general switches	   */
int    ct0, ct1, ct2, ct3;			/* general counters	   */
int    ex, wy;					/* hiplot coordinates	   */
double nu0, nu1, nu2, nu3;			/* general numeric values  */
double mf;					/* mouse increment	   */
double sm[TS];					/* selfconsistent nbrhd    */
double sn[TS];					/* selfconsistent nbrhd    */
double bx[BD];					/* bernstein factor        */
double bp[BD];
double pv[10];					/* polynomial vector 	   */
double fpa, fpb, fpc;				/* fixed points		   */

int    cc, dd;
double blpr[KK];			/* probability from blocks	*/


Inst 	xprg[10][XLEN];			
Inst   	rprg[16384];			/* rec program array */
char   	cstr[CLEN];			/* console rec program */
char   	cprg[CLEN];			/* edit stored program */
char	idef [1][30] = {"((z!100!0r:;)1r2r(0r:;);) a"};

extern int rec_c(char *z, Inst rprg, int l, struct fptbl dtbl[]);

long int r_intpar;

lcau2tmain()
{
int  i, j, i0, i1, i2, i3;
int  more='r', sc=0;
char a, b, c;

for (i=0; i<TS; i++) ttrule[i]='0';
for (i=0; i<KK+1; i++) for (j=0; j<TS-2; j++) srule[i][j]='0';

ru=6; ru0=0; ru1=0; ru2=0; ru3=0;
ct0=ct1=ct2=ct3=0;					/* counters	       */
sw0=sw1=sw2=sw3=0;					/* switches	       */
co=1;							/* normal connectivity */
dn=1;							/* one dimension       */
bi=0;
rij=2; rdi=0; rdj=1;
rp=rq=0; cstr[rp]=0;
li=SL/2; lj=0;						/* line cursor         */
pr=500;	pl=500;						/* probability, mils */
mf=0.05;
for (i=0; i<BD; i++) bx[i]=1.0;				/* bernstein factor */

    videopalette(WHCYMAG);				/* white/cyan/magenta */

    tuto();
    while (!kbdst()) rand();				/* wait for keypress */
    kbdin();						/* ignore it */
    videomode(T80X25);
    videoscroll(0,0,CR,71,0,3);				/* menu on blue background */
    videoscroll(CL+8,0,24,71,0,3);
    i=(rand()%NX);
    di=i%DROWS;
    dj=(i/DROWS==1?DCOLS:1);
    xtoasc(i);
    auxblnk();
    ranlin();						/* random initial array */

    while (more!='n') {					/* execute multiple runs */
    rmenu();
    lmenu();
    while (0<1) {					/* set up one run */
    c=kbdin();
    if (c=='g') break;					/* go draw graph */
    if (c=='q') more='n';				/* quit for good */
    if (more=='n') break;
    switch (c) {

        case Kf1:					/* F1 = fetch stored rule */
	    dij=dmenu(Kf1);
	    xtoasc(lim(0,dij,NX-1));
	    rmenu();
	    lmenu();
	    break;
        case Kf2:					/* F2 = rec program */
	    rij=lim(1,recdem(Kf2),NY)-1;
	    for (i=0; i<CLEN; i++) cstr[i]=recrule[rij][i];
	    rmenu();
	    lmenu();
	    break;
        case Kf3:					/* f3 = edit rec program */
	    edrec(cstr); 
	    rmenu();
	    lmenu();
	    break;
        case Kf4:					/* f4 = execute rec program */
	    if (rec_c(cstr,rprg,PLEN,dtbl)==1) rec_x(rprg);
	    break;
        case Kf5:					/* f5 = execute demo program */
	    if (rec_c(&recrule[rdi],rprg,PLEN,dtbl)==1) rec_x(rprg);
	    break;

        case '!': sc=7; break;
	case '*': sc=2; break;				/* white on red */
	case '+': sc=1; break;				/* white screen */
	case '-': sc=0; break;				/* blue screen */
	case '@':					/* numbered tot rule */
	    nutoto(numin(0));
	    totoasc();
	    rmenu();
	    videocursor(0,CM,0);
	    break;
	case '$':					/* dozen totalistics */
	    j=numin(0);
	    for (i=0; i<12; i++) {
	      nutoto(j+i);
	      totoasc();
	      ranlin();
	      evolve(sc);
	      }
            videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
	case 'T':					/* totalistic rule */
	    xblnk();
	    tmenu();
	    edtrule();
	    totoasc();
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	      ascrule[i0][i1][i2][i3]=ttrule[i0+i1+i2+i3];
	    videocursor(0,CM,0);
	    rmenu();
	    break;
	case 't':					/* probability calculation */
	    edtri();
	    rmenu();
	    lmenu();
	    break;
        case 'r':					/* edit rule */	
	    xblnk();
	    edrule();
	    videocursor(0,CM,0);
	    rmenu();
	    break;
        case 'l':					/* edit cell string */
	    xblnk();
	    edline(8,40);
	    videocursor(0,CM,0);
	    lmenu();
	    break;
	case 'D':					/* run through samples */
	    for (i=0; i<NX; i++) {
	      xmenu(i);
	      xtoasc(i);
	      ranlin();
	      evolve(sc);
	      }
	    videomode(T80X25);
	    rmenu();
	    break;
	case 'd':					/* de Bruijn diagram */
	    edijn();
	    rmenu();
	    lmenu();
	    break;
	case 'e':					/* Garden of Eden */
	    edard();
	    rmenu();
	    lmenu();
	    break;
        case 'u':					/* sparse init arry */
	    xblnk();
	    for (i=0; i<AL; i++) arr1[i]=0;
	    arr1[AL/4]=1;
            arr1[AL/2]=1;
            arr1[(3*AL)/4]=1;
            arr1[(3*AL)/4+2]=1;
	    lmenu();
            break;
	case 'w':					/* Wolfram rule # */
	    i=numin(0);
	    wmenu(i);
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	      {ascrule[i0][i1][i2][i3]='0'+i%KK; i/=KK;}
	    rmenu();
	    break;
	case 'X':					/* random rule */
	    xblnk();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    for (i3=0; i3<KK; i3++) {
	      if ((KK*(KK*i0+i1)+i2)%4 == 0) i=rand();
	      ascrule[i0][i1][i2][i3]='0'+i%KK;
	      i/=KK;
	    }}}}
	    rmenu();
	    break;
	case 'x':					/* random rule */
	    xblnk();
	    i=rand();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    for (i3=0; i3<KK; i3++) {
	      if (i == 0) i=rand();
	      if (auxrule[i0][i1][i2][i3]==' ') {
		ascrule[i0][i1][i2][i3]='0'+i%KK;
		i/=KK;
		}
	      }}}}
	    rmenu();
	    break;
	case 'y':					/* random line */
	    xblnk();
	    ranlin();
            lmenu();
	    break;
	case 'Y':					/* symmetrize rule */
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    for (i3=0; i3<KK; i3++) {
	    ascrule[i3][i2][i1][i0]=ascrule[i0][i1][i2][i3];      
	    }}}}
	    break;
	case 'B':					/* begin barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[0][a-'0'][b-'0'][a-'0']=a;
	    ascrule[1][a-'0'][b-'0'][a-'0']=a;
	    ascrule[2][a-'0'][b-'0'][a-'0']=a;
	    rmenu();
	    break;
	case 'E':					/* end barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][0]=b;
	    ascrule[a-'0'][a-'0'][a-'0'][1]=b;
	    ascrule[a-'0'][a-'0'][a-'0'][2]=b;
	    rmenu();
	    break;
	case 'L':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0']=c;
	    rmenu();
	    break;
	case 'R':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0']=a;
	    rmenu();
	    break;
	case 'S':					/* still life link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0']=b;
	    rmenu();
	    break;
	case '.':					/* one cycle of evolution */
	    asctobin();
	    onegen(AL);
	    lmenu();
	    break;
	case '=':
	    for (i=1; i<8;  i++) {
	    for (j=0; j<40; j++) arr1[40*i+j]=arr1[j];}
	    lmenu();
	    break;
	case '~':
	    for (i=1; i<16;  i++) {
	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];}
	    lmenu();
	    break;
        default: break;
        }
    }
    if (more=='n') break;
    do {
    evolve(sc);
    videocursor(0,0,0);
    scrstr("?");
    videocursor(0,0,34);
    scrstr("y/n/cr");
    more=kbdin();
    } while (more=='\015');
    videomode(T80X25);					/* reset the screen */
    if (more=='n') break;
    }
  videomode(T80X25);}    

/* edit the rule */
edrule() {char c;
  while (0<1) {
    cscrrul();
    videocursor(0,5,ru);
    c=kbdin();
    if (c=='\015') break;				/* carriage return exits */
    switch (c) {
    case '0': case '1':					/* state */
      ascrule[ru0][ru1][ru2][ru3]=c;
      videocattr(0,c,3,1);
      rupl();
      break;
    case '\011':					/* tab = next quad */
      ru3=0; ru2=0; ru1=0; ru0++;
      if (ru0==KK) ru0=0;
      ru=KK*KK*KK*ru0+6;
      break;
    case ' ': case '\315': rupl(); break;				 /* advance */
    case '\010': case '\313': rumi(); break;				 /* backspace */
    case '\013': case '\310': auxrule[ru0][ru1][ru2][ru3]='+'; rupl(); break; /* up arrow */
    case '\012': case '\320': auxrule[ru0][ru1][ru2][ru3]=' '; rupl(); break; /* down arrow */
    case 'z': asczero();
    default: break;
    }
  }
}

/* advance the rule cursor */
rupl() {
  ru3++;
  if (ru3==KK) {ru3=0; ru2++;}
  if (ru2==KK) {ru2=0; ru1++;}
  if (ru1==KK) {ru1=0; ru0++;}
  if (ru0==KK) {ru0=0; }
  ru=KK*(KK*(KK*ru0+ru1)+ru2)+ru3+6;
  }

/* retract the rule cursor */
rumi() {
  if (ru3!=0) ru3--; else {ru3=KK-1;
  if (ru2!=0) ru2--; else {ru2=KK-1;
  if (ru1!=0) ru1--; else {ru1=KK-1;
  if (ru0!=0) ru0--; else {ru0=KK-1;
     }}}}
  ru=KK*(KK*(KK*ru0+ru1)+ru2)+ru3+6;
  }

/* edit totalistic rule */
edtrule() {char c; int  i, k;
  i=0;
  while (i<TS) {
    c=ttrule[i];
    videocursor(0,3,56+i);
    videocattr(0,c,3,1);
    c=kbdin();
    if (c=='\015') break;
    switch (c) {
      case '0': case '1':				/* state */
        ttrule[i]=c;
        videocattr(0,c,CR,1);
        i++;
        break;
      case ' ': case '\315':			/* space = advance */
        i++;
        break;
      case '\010': case '\313':			/* backspace */
        if (i!=0) i--;
	break;
      case 'x':					/* random selection */
	for (i=0; i<TS; i++) {
	  if (i%8==0) k=rand();
	  ttrule[i]='0'+k%KK; k/=KK;}
	i=0;
	tscrrul();
	break;
      default: break;
      }
  }
}

/* set up a grid for the unit square */
pgrid() {int i, j, k, l;
k=KK*KK+1;
l=195/(k-1);
for (i=0; i<k; i++) {
for (j=0; j<k; j++) {
videodot(l*i,l*j,2);
}}
}


/* map the automaton state into the unit square */
void pprob() {int i, j; float x, y, z, k;
i=AL/2; k=(float)KK; z=(float)((197/(KK*KK))*KK*KK);
x=0.0; for (j=1; j<25; j++) x=(x+(float)arr1[i+j])/k;
y=0.0; for (j=1; j<25; j++) y=(y+(float)arr1[i-j])/k;
x=(x+(float)arr1[i])/k;
videodot(100-(int)(z*(1.0-y)),1+(int)(z*x),1);
}


/* edit the line of cells */
edline(nr,nc) int nr, nc; {
char c;
int  i, j, k, l, m;
int  aa, bb, cc, dd;
int  ww, xx, yy, zz;
int  ll;

  m=0; /* refresh trigger */
  cscrrul();
  videocursor(0,19,0);
  scrstr("insert states with 0, 1; move cursor");
  videocursor(0,20,0);
  scrstr("with keyboard arrows, space or backspace.");
  videocursor(0,21,0);
  scrstr("carriage return exits");
  while (0<1) {
  ll=lj;
  videocursor(0,lj+LROW,li);
  c=kbdin();
  if (c == '\015') {videoscroll(19,0,24,70,0,3); break;}
  switch (c) {
  case '0': case '1':					/* enter state */
    arr1[nc*lj+li]=c-'0';
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\012': case '\320':				/* downwards */
    lj++; if (lj>=nr) lj=0;
    break;
  case '\013': case '\310':				/* upwards */
    if (lj>0) lj--; else lj=nr-1;
    break;
  case '\014': case '\315': case ' ':			/* forward */
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\010': case '\313':				/* backward */
    if (li>0) li--;
      else {li=nc-1; if (lj>0) lj--; else lj=nr-1;}
    break;
  case '<':						/* left margin */
    li=0;
    break;
  case '>':						/* right margin */
    li=nc-1;
    break;
  case 'z':						/* clear single row */
    for (k=0; k<nc; k++) arr1[nc*lj+k]=0;
    break;
  case 'Z':						/* clear full array */
    for (k=0; k<AL; k++) arr1[k]=0;
    lmenu();
    break;
  case '^':						/* copy line above */
    for (k=0; k<nc; k++)
      arr1[nc*lj+k]=arr1[nc*(lj-1)+k];
    break;
  case 'x':						/* unmark transitions */
    auxblnk();
    cscrrul();
    break;				
  case 'q':						/* clean up the area */
    ulmenu(3);
    break;
  case '=':		/* insert point transition in ascrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    c=arr1[yy+nc]+'0';
    ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
    rmenu(); 
    cscrrul();
    break;
  case '+':		/* set marker for transition in auxrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]='+';
    rmenu(); 
    cscrrul();
    break;
  case '-':		/* remove marker for transition from auxrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=' ';
    rmenu(); 
    cscrrul();
    break;
  case '*':		/* insert transitions for whole line */
    if (lj==0) break;
    for (i=0; i<nc; i++) {
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      c=arr1[yy+nc]+'0';
      ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
      auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
      }
    rmenu(); 
    cscrrul();
    break;
  case '.':		/* point evolution */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    arr1[yy+nc]=ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]-'0';
    break;
  case '?':		/* evolution of whole line */
    if (lj==nr-1) {for (i=0; i<YL-nc; i++) arr1[i]=arr1[i+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      arr1[yy+nc]=ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]-'0';
      }
    break;
  case '/':		/* conditional evolution */
    if (lj==nr-1) {for (k=0; k<YL-nc; k++) arr1[k]=arr1[k+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      j=0;
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx]; if (aa>KK-1) j=KK;
      bb=arr1[yy]; if (bb>KK-1) j=KK;
      cc=arr1[zz]; if (cc>KK-1) j=KK;
      if (auxrule[aa][bb][cc][dd]==' ') j=KK;
      if (j==0) j=ascrule[aa][bb][cc][dd]-'0';
      arr1[yy+nc]=j;
      }
    break;
  case 'c':		/* test consistency everywhere */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      yy=i+nc*(j-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx];
      bb=arr1[yy];
      cc=arr1[zz];
      c=arr1[yy+nc]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,c=='0'?'.':c,c==ascrule[aa][bb][cc][dd]?7:4,1);
      }}
    m=1;
    break;
  case 'C':		/* test consistency for marked neighborhoods */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      yy=i+nc*(j-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx];
      bb=arr1[yy];
      cc=arr1[zz];
      c=arr1[yy+nc]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,c=='0'?'.':c,
        (c==ascrule[aa][bb][cc][dd] && auxrule[aa][bb][cc][dd]!=' ')?7:4,1);
      }}
    m=1;
    break;
  default: break;
      } /* end switch */
  if (m==0) for (k=0; k<nc; k++) {
    videocursor(0,ll+LROW,k);
    l=arr1[nc*ll+k]+'0';
    videoputc(l=='0'?'.':l,1);
    }
  m=0;
  }     /* end while */
}       /* end edline */

/* calculate index, or rather the neighbors, in edline. */
linix(xx,i,j,nc) int xx[RR+1], i, j, nc; {int k, x;
  xx[(RR-1)/2]=x=i+nc*(j-1);
  if ((i==0)||(i==1)) xx[0]=x+nc-2; else xx[0]=x-2;
  if (i==0) xx[1]=x+nc-1; else xx[1]=x-1;
  if (i==nc-1) xx[3]=x-nc+1; else xx[3]=x+1;
  if ((i==nc-1)||(i==nc-2)) xx[4]=x-nc+2; else xx[4]=x+2;
  for (k=0; k<RR; k++) xx[k]=arr1[xx[k]];
  xx[RR]=arr1[x+nc];
}


/* display a screen of evolution */
evolve(sc) int sc; {
int i, j, k, l;
  asctobin();
  for (j=0; j<200; j++) {		/* evolve for 200 generations */
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
    }
  } 

/* copy ascrule over to binrul */
asctobin() {int i0, i1, i2, i3;
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++) {
      binrule[i0][i1][i2][i3]=ascrule[i0][i1][i2][i3]-'0';
      }
}

/* evolution for one generation */

onegen(j) int j; {int i, k, i0, i1, i2, i3; long int ii;
switch (dn) {

case 1:
if (j>3) {
  arr2[0]=binrule[arr1[j-2]][arr1[j-1]][arr1[0]][arr1[1]];
  arr2[1]=binrule[arr1[j-1]][arr1[0]][arr1[1]][arr1[2]];
  for (i=2; i<j-1; i++) 
    arr2[i]=binrule[arr1[i-2]][arr1[i-1]][arr1[i]][arr1[i+1]];
  arr2[j-2]=binrule[arr1[j-4]][arr1[j-3]][arr1[j-2]][arr1[j-1]];
  arr2[j-1]=binrule[arr1[j-3]][arr1[j-2]][arr1[j-1]][arr1[0]];
} else if (j==3) {
  arr2[0]=binrule[arr1[1]][arr1[2]][arr1[0]][arr1[1]]; 
  arr2[1]=binrule[arr1[2]][arr1[0]][arr1[1]][arr1[2]]; 
  arr2[2]=binrule[arr1[0]][arr1[1]][arr1[2]][arr1[0]]; 
} else if (j==2) {
  arr2[0]=binrule[arr1[0]][arr1[1]][arr1[0]][arr1[1]]; 
  arr2[1]=binrule[arr1[1]][arr1[0]][arr1[1]][arr1[0]]; 
} else if (j==1) {
  arr2[0]=binrule[arr1[0]][arr1[0]][arr1[0]][arr1[0]]; 
}
if (co==1) for (ii=0; ii<j; ii++) arr1[(int)ii]=arr2[(int)(ii)];
if (co==0) for (ii=0; ii<j; ii++) arr1[(int)ii]=arr2[(int)((7*ii)%j)];
if (co==2) for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];}
if (co==3) for (ii=0; ii<j; ii+=3) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];
				 arr1[(int)ii+2]=arr2[(int)((7*ii+2)%j)];}
if (co==4) for (ii=0; ii<j; ii+=4) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];
				 arr1[(int)ii+2]=arr2[(int)((7*ii+2)%j)];
				 arr1[(int)ii+3]=arr2[(int)((7*ii+3)%j)];}

if (co==5) {for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((11*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((11*ii+1)%j)];}
	    for (ii=0; ii<j; ii+=8) {
		k=arr1[(int)ii];
		arr1[(int)ii]=arr1[(int)((ii+2)%j)];
		arr1[(int)((ii+2)%j)]=k;
		k=arr1[(int)((ii+1)%j)];
		arr1[(int)((ii+1)%j)]=arr1[(int)((ii+4)%j)];
		arr1[(int)((ii+4)%j)]=k;
		k=arr1[(int)((ii+3)%j)];
		arr1[(int)((ii+3)%j)]=arr1[(int)((ii+5)%j)];
		arr1[(int)((ii+5)%j)]=k;
                }
	    }
if (co==6) {for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((11*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((11*ii+1)%j)];}
	    for (ii=0; ii<j; ii+=16) {
		k=arr1[(int)ii];
		arr1[(int)ii]=arr1[(int)((ii+2)%j)];
		arr1[(int)((ii+2)%j)]=k;
		k=arr1[(int)((ii+1)%j)];
		arr1[(int)((ii+1)%j)]=arr1[(int)((ii+4)%j)];
		arr1[(int)((ii+4)%j)]=k;
		k=arr1[(int)((ii+3)%j)];
		arr1[(int)((ii+3)%j)]=arr1[(int)((ii+5)%j)];
		arr1[(int)((ii+5)%j)]=k;

		k=arr1[(int)((ii+8)%j)];
		arr1[(int)((ii+8)%j)]=arr1[(int)((ii+10)%j)];
		arr1[(int)((ii+10)%j)]=k;
		k=arr1[(int)((ii+9)%j)];
		arr1[(int)((ii+9)%j)]=arr1[(int)((ii+15)%j)];
		arr1[(int)((ii+15)%j)]=k;
		k=arr1[(int)((ii+11)%j)];
		arr1[(int)((ii+11)%j)]=arr1[(int)((ii+13)%j)];
		arr1[(int)((ii+13)%j)]=k;
		k=arr1[(int)((ii+12)%j)];
		arr1[(int)((ii+12)%j)]=arr1[(int)((ii+14)%j)];
		arr1[(int)((ii+14)%j)]=k;
		}
	    } break;

/* Moore (2,1/2) in two dimensions */

case 2: if (j<16384) break;
	  for (i0=0; i0<128; i0++) for (i1=0; i1<128; i1++) 
	    arr2[ix2(i0,i1)]=binrule
	      [arr1[ix2(i0,i1)]][arr1[ix2(i0,i1+1)]]
	      [arr1[ix2(i0+1,i1)]][arr1[ix2(i0+1,i1+1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* the corner cell on a cube */

case 3: if (j<16384) break;
	  for (i0=0; i0<32; i0++)
	  for (i1=0; i1<32; i1++)
	  for (i2=0; i2<16; i2++)
	    arr2[ix3(i0,i1,i2)]=binrule
	      [arr1[ix3(i0+1,i1,i2)]][arr1[ix3(i0,i1+1,i2)]]
	      [arr1[ix3(i0,i1,i2+1)]][arr1[ix3(i0,i1,i2)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* the corner cell on a hypercube */

case 4: if (j<16384) break;
	  for (i0=0; i0<16; i0++) for (i1=0; i1<8; i1++) 
	  for (i2=0; i2<16; i2++) for (i3=0; i3<8; i3++) 
	    arr2[ix4(i0,i1,i2,i3)]=binrule
	      [arr1[ix4(i0+1,i1,i2,i3)]][arr1[ix4(i0,i1+1,i2,i3)]]
	      [arr1[ix4(i0,i1,i2+1,i3)]][arr1[ix4(i0,i1,i2,i3+1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* von Neumann (2,1) in two dimensions without the center cell */

case 5: if (j<16384) break;
	  for (i0=0; i0<128; i0++) for (i1=0; i1<128; i1++) 
	    arr2[ix2(i0,i1)]=binrule
	      [arr1[ix2(i0,i1-1)]][arr1[ix2(i0,i1+1)]]
	      [arr1[ix2(i0-1,i1)]][arr1[ix2(i0+1,i1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* the center cell of a tetrahedron */

case 6: if (j<16384) break;
	  for (i0=0; i0<32; i0++)
	  for (i1=0; i1<32; i1++)
	  for (i2=0; i2<16; i2++)
	    arr2[ix3(i0,i1,i2)]=binrule
	      [arr1[ix3(i0,i1+1,i2-1)]][arr1[ix3(i0,i1+1,i2+1)]]
	      [arr1[ix3(i0+1,i1-1,i2)]][arr1[ix3(i0-1,i1-1,i2)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

default: break;}
}

/* index two dimensions */
int ix2(i,j) int i, j; {return (128+i)%128+128*((128+j)%128);}

/* index three dimensions */
int ix3(i,j,k) int i, j, k; {return (32+i)%32+ 32*((32+j)%32+ 32*((16+k)%16 ));}

/* index four dimensions */
int ix4(i,j,k,l) int i, j, k, l; {
return (16+i)%16+ 16*( (8+j)%8+ 8*( (16+k)%16+ 16*( (8+l)%8 )));}

/* generate a random line of cells in arr1 */
ranlin() {int i, j;
for (i=0; i<AL; i++) {if (i%8 == 0) j=rand(); arr1[i]=j%KK; j/=KK;}}

/* generate line of cells in arr1 with n mils of 1's */
rprob(n) int n; {int i; for (i=0; i<AL; i++) arr1[i]=(rand()%1000<n)?1:0;}

/* generate line of n cells in character array a with p mils of 1's */
ranchay(a,p,n) char *a; int p, n; {int i;
  for (i=0; i<n; i++) a[i]=(rand()%1000<p)?'1':'0';}

/* uniform line menu */
ulmenu(l) int l; {int i, j, k;
  for (j=0; j*SL<YL; j++) {
  for (i=0; i<SL;    i++) {
    videocursor(0,LROW+j,i);
    k=arr1[j*SL+i];
    videocattr(0,k==0?'.':k+'0',l,1);
    }}
  }

/* tutorial and Help screen */
tuto() {
    videomode(T80X25);
    videocursor(0,2,0);
    scrstr("<Copyright (C) 1987, 1988, 1989 - H.V. McIntosh, G. Cisneros S.>");
    videocursor(0,4,0);
    scrstr("           **** LIFE in One Dimension ****");
    videocursor(0,6,0);
    scrstr("Two States - Black(0), Cyan(1).");
    videocursor(0,8,0);
    scrstr("3 half neighbors - two on each side, four altogether.");
    videocursor(0,10,0);
    scrstr("Complete transition rule - random, edited, or stored.");
    videocursor(0,12,0);
    scrstr("Initial Cellular Array - random, edited, or patterned.");
    videocursor(0,14,0);
    scrstr("Submenus in options t(probabilities) and d(de Bruijn)");
    videocursor(0,15,0);
    scrstr("will be displayed in response to typing ?");
    videocursor(0,17,0);
    scrstr("Use any key to terminate a display in progress.");
    videocursor(0,21,0);
    scrstr("Now, ... press any key to continue.");
}

/* rule menu */
rmenu() {
    videocursor(0,CR-4,0);
    scrstr("      0000000011111111");
    videocursor(0,CR-3,0);
    scrstr("      0000111100001111");
    videocursor(0,CR-2,0);
    scrstr("      0011001100110011");
    videocursor(0,CR-1,0);
    scrstr("      0101010101010101");
    videocursor(0,CR,0);
    scrstr("Rule: ");
    scrrul();
    videocursor(0,CR,42);
    printf("      ");
    videocursor(0,CR,42);
    if (istot()==1) printf("(T-%2d)",totonu(0));
    videocursor(0,CM,0);
    scrstr("    r(rule), l(line), #(stored), g(graph), q(quit), t(prob),");
    videocursor(0,CM+1,0);
    scrstr("         x(rand rule), y(rand line), u(unit line), d(deBruijn)");
    videocursor(0,CM+2,0);
    scrstr("         @nn(tot/rule), $nn(12 tot/rules), T(ed tot/rule).");
    videocursor(0,CM,0);
    }

/* totalistic rule menu*/
tmenu() {
    videocursor(0,CR-3,50);
    scrstr("      0...1");
    videocursor(0,CR-2,50);
    scrstr("rule: ");
    tscrrul();
    videocursor(0,CR-2,56);
    }

/* line menu */
lmenu() {int i, j;
  for (j=0; 40*j<YL; j++) {
    videocursor(0,CL+j,0);
    for (i=0; i<40; i++) videoputc('0'+arr1[40*j+i],1);
    }
  videocursor(0,CM,0);
  }

/* display Wolfram rule number */
wmenu(n) int n;
{char xx[6]; int i, nn;  
    nn=sprintf(xx,"%5d",n);
    videocursor(0,2,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,2,40); }

/* display rule number */
xmenu(n) int n;
{char xx[4]; int i, nn;  
    nn=sprintf(xx,"%3d",n);
    videocursor(0,1,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,1,40); }

/* clear screen space for rule numbers */
xblnk() {
  videocursor(0,CR-4,40); scrstr("     ");
  videocursor(0,CR-3,40); scrstr("     ");
  videocursor(0,CR,0); }

/* clear auxrule to blanks */
auxblnk() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  auxrule[i0][i1][i2][i3]=' ';
}

/* clear ascrule to zeroes */
asczero() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]='0';
}

/* find the totalistic rule number of a known ascii totalistic rule */
void asctotot() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
      ttrule[i0+i1+i2+i3] = ascrule[i0][i1][i2][i3];
}

/* generate a random core for a planar array */
ranplacor() {int i, j;
  for (i=60; i<69; i++) for (j=60; j<69; j++)  arr1[ix2(i,j)] = random()%KK;}

/* generate a random core for a solid array */
ransolcor() {int i, j, k;
  for (i=14; i<19; i++) for (j=14; j<19; j++) for (k=7; k<10; k++) 
	 arr1[ix3(i,j,k)] = random()%KK;}

/* make all of arr1 equal to zero */
void zerline() {int i; for (i=0; i<AL; i++) arr1[i]=0;}

/* put a single i-cell on a line of j-cells */
void putionj(i,j) int i, j; 
{int k; for (k=0; k<AL; k++) arr1[k]=j; arr1[AL/2]=i;}

/* copy saved rule #n into active rule */
xtoasc(n) int n; { int i0, i1, i2, i3;
xmenu(n+1);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]=ixrule[n][((i0*KK+i1)*KK+i2)*KK+i3];
}

/* change totalistic rule to general rule */
totoasc() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]=ttrule[i0+i1+i2+i3];
}

/* change decimal totalistic rule to sum values */
nutoto(x) int x; {int i; for (i=0; i<TS; i++) {ttrule[i]=x%KK+'0'; x/=KK;}}

/* change sum values to decimal totalistic rule */
int totonu(i) int i; {int r;
 r=0; if (i<TS) r=(ttrule[i]-'0')+KK*totonu(i+1);
  return r; }

/* test whether a rule is semi totalistic */
int isstot() {int st[KK+1][TS-2], i0, i1, i2, i3, l;
    l=1;
    st[0][0]=ascrule[0][0][0][0];
    st[0][1]=ascrule[0][1][0][0];
    st[0][2]=ascrule[0][1][1][0];
    st[1][0]=ascrule[0][0][1][0];
    st[1][1]=ascrule[1][0][1][0];
    st[1][2]=ascrule[1][0][1][1];
    st[2][0]=ascrule[0][1][1][0];
    st[2][1]=ascrule[1][1][1][0];
    st[2][2]=ascrule[1][1][1][1];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
      if (ascrule[i0][i1][i2][i3] !=st[i1+i2][i0+i3]) l=0;      
    return l; }

/* test whether a rule is totalistic */

int istot() {int tr[5], i0, i1, i2, i3, l;
    l=1;
    tr[0]=ascrule[0][0][0][0];
    tr[1]=ascrule[1][0][0][0];
    tr[2]=ascrule[1][1][0][0];
    tr[3]=ascrule[1][1][1][0];
    tr[4]=ascrule[1][1][1][1];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
      if (ascrule[i0][i1][i2][i3] !=tr[i0+i1+i2+i3]) l=0;      
    return l; }

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}




/*                   insercion            */


/* clear auxrule to zeroes */
auxzero() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
  auxrule1[i0][i1][i2]=0;
}


/* change semitotalistic rule to general rule */
sttoasc() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]=strule[i1+i2][i0+i3];
}

/* clear strule to zeroes */
stzero() {int i, j;
  for (i=0; i<KK+1; i++) for (j=0; j<SS-1; j++)  strule[i][j]='0';
}

/* clear gf2rule to zeroes */
gfzero() {int i; for (i=0; i<13; i++) gf2rule[i]='0';}


/* generate a random line of cells in arr1 with assigned probabilities */
ranblin(p1,p2) double p1, p2; {
  int    i, c, i1;
  double p;
  p=p1+p2;
  i1=(int)((1000.0*p1)/p);
   for (i=0; i<AL; i++) {
    c=random()%1000;
    if (c<i1) arr1[i]=0; else arr1[i]=1;
    }
}


/* change a GF2 rule to general rule */
void gftoasc() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++) 
  ascrule1[i0][i1][i2]=
    (i0*((gf2rule[0]-'0')*i0+(gf2rule[9]-'0'))+
     i0*(gf2rule[1]-'0')*i1+
     i0*(gf2rule[2]-'0')*i2+
     i1*((gf2rule[3]-'0')*i0+(gf2rule[10]-'0'))+
     i1*(gf2rule[4]-'0')*i1+
     i1*(gf2rule[5]-'0')*i2+
     i2*((gf2rule[6]-'0')*i0+(gf2rule[11]-'0'))+
     i2*(gf2rule[7]-'0')*i1+
     i2*(gf2rule[8]-'0')*i2+
    (gf2rule[12]-'0') )%KK+'0';
}

/* display the rule number on the screen */
scrrul() {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  videoputc(ascrule[i0][i1][i2][i3],1);      
  }

/* display the rule number on the screen in glorious technicolor */
cscrrul() {int i, i0, i1, i2, i3;
  i=6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    {
    videocursor(0,5,i++);
    videocattr(0,ascrule[i0][i1][i2][i3],auxrule[i0][i1][i2][i3]==' '?3:6,1);
    }
  }

/* turn ascrule into its Wolfram number and place on screen at (i,j) */
woruno(i,j) int i, j; {
char xx[12];
int  k, l, i0, i1, i2, i3;
  k=0;
  for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
  for (i2=KK-1; i2>=0; i2--)
  for (i3=KK-1; i3>=0; i3--)
    k=KK*k+(ascrule[i0][i1][i2][i3]-'0');
  k=sprintf(xx,"%10u",k);
  videocursor(0,i,j);
  for (l=0; l<k; l++) videoputc(xx[l],1);
}

/* display semitotalistic rule number by sum */
sscrrul(j) int j; {int i; for (i=0; i<TS-2; i++) videoputc(srule[j][i],1); }

/* display totalistic rule number by sum */
tscrrul() {int i; for (i=0; i<TS; i++) videoputc(ttrule[i],1); }

/* write a string in graphics mode */
scrstr(s) char *s; {for (; *s != '\0'; s++) videoputc(*s,1); }

/* report Wolfram rule number or variant */
void wolnum(ii,c) int ii[KK][KK][KK][KK]; char c; {
int  k, i0, i1, i2, i3;
  k=0;
  switch (c) {
    case 'w':
      /*for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)*/

      for (i0=0; i0<KK; i0++)
      for (i1=0; i1<KK; i1++)
      for (i2=0; i2<KK; i2++)
      for (i3=0; i3<KK; i3++)
      ii[i0][i1][i2][i3]=(ascrule[i0][i1][i2][i3]-'0');
      break;
    case 'c':
      /*for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)*/

      for (i0=0; i0<KK; i0++)
      for (i1=0; i1<KK; i1++)
      for (i2=0; i2<KK; i2++)
      for (i3=0; i3<KK; i3++)
 ii[i0][i1][i2][i3]=((ascrule[(i0+1)%KK][(i1+1)%KK][(i2+1)%KK][(i3+1)%KK]-'0')+1)%KK;
      break;
    case 'r':
      /*for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)*/
 
      for (i0=0; i0<KK; i0++)
      for (i1=0; i1<KK; i1++)
      for (i2=0; i2<KK; i2++)
      for (i3=0; i3<KK; i3++)
 ii[i0][i1][i2][i3]=(ascrule[i3][i2][i1][i0]-'0');
      break;
    case 'b':
     /* for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)*/

       for (i0=0; i0<KK; i0++)
      for (i1=0; i1<KK; i1++)
      for (i2=0; i2<KK; i2++)
      for (i3=0; i3<KK; i3++)
 ii[i0][i1][i2][i3]=((ascrule[(i3+1)%KK][(i2+1)%KK][(i1+1)%KK][(i0+1)%KK]-'0')+1)%KK;
      break;
    default: k=0; break;}
    }

txtmode() {videomode(T80X25); rmenu(); lmenu();}
int grfmode() {videomode(COLGRAF);}

/* place time and date on screen */ 
int tidat() {long cl[1]; char *ctime(); time(cl); printf("%26s",ctime(cl));}

/* keyboard status */
int kbdst() {return(bdos(11) & 0xFF);}

/* direct keyboard input, no echo */
 int kbdin() {int c;
  if ((c = bdos(7) & 0xFF) == '\0') c = (bdos(7) & 0xFF) | 0x80;
  return(c);
}

/* turn ASCII character into hexadecimal value */
int nibbl(c) int c; {
  if (c>='0'&&c<='9') return (c-'0');
  if (c>='a'&&c<='f') return (c-'a'+10);
  if (c>='A'&&c<='F') return (c-'A'+10);
  return 0;
}

/* turn a nibble into an ASCII character */
int nibasc(i) int i; {if (i<10) return '0'+i; else return 'A'+i-10;}

/* read number from keyboard */
int numin(n) int n; {char c;
  c=kbdin();
  if (c>='0'&&c<='9') return(numin(10*n+(c-'0'))); else return(n);
}

void rule2t(){
    xtoasc((int)(random()%NX));
    ranlin();					/* random initial array */
}

void savearry()  {int i; for (i=0; i<AL; i++) arr3[i]=arr1[i];}
void renewarry() {int i; for (i=0; i<AL; i++) arr1[i]=arr3[i];}

int  bdos(){} 


/* report Wolfram rule number or variant */
void  chrucopy(char ii[KK][KK][KK][KK],char jj[KK][KK][KK][KK],char c)
{int  i0, i1, i2, i3;
  switch (c) {
    case 'w':
      for (i0=0; i0<KK; i0++) 
      for (i1=0; i1<KK; i1++) 
      for (i2=0; i2<KK; i2++)
      for (i3=0; i3<KK; i3++)  
        ii[i0][i1][i2][i3]=jj[i0][i1][i2][i3];
      break;
    case 'r':
      for (i0=0; i0<KK; i0++) 
      for (i1=0; i1<KK; i1++) 
      for (i2=0; i2<KK; i2++) 
      for (i3=0; i3<KK; i3++) 
         ii[i0][i1][i2][i3]=jj[i3][i2][i1][i0];
      break;
    case 'y':
      for (i0=0; i0<KK; i0++) 
      for (i1=0; i1<KK; i1++) 
      for (i2=0; i2<KK; i2++) 
      for (i3=0; i3<KK; i3++) 
         ii[i0][i1][i2][i3]=ycc(jj[cyy(i0)][cyy(i1)][cyy(i2)][cyy(i3)]);
      break;
    default: break;}
       }

/* end nxlcau2t.c */
