
/* nxprob2t.c						*/
/* program for LCA2T option 't'				*/
/* calculate probabilities related to evolution		*/
/* Harold V. McIntosh, 10 August 1987			*/
/* 29 February 1988 - adapted from LCA21 to LCA22	*/
/* 11 March 1989 - adapted from LCAU23 to LCAU24	*/
/* 16 March 1989 - draw diagonal in M panel		*/
/* 18 March 1989 - Options n,N - totalistic statistics	*/
/* 15 September 1989 - entropy calculations		*/
/* 14 October 1989 - adapted to radius 3/2		*/
/* 10 February 1992 - Houston Instrument plotter option	*/

/* references:							*/
/*								*/
/* W. John Wilbur, David J. Lipman and Shihab A. Shamma		*/
/* On the prediction of local patterns in cellular automata	*/
/* Physica 19D 397-410 (1986)					*/ 
/*								*/
/* Howard A. Gutowitz, Jonathan D. Victor and Bruce W. Knight	*/
/* Local structure theory for cellular automata			*/
/* Physica 28D 18-48 (1987)					*/

/*  Copyright (C) 1987  */
/*  Copyright (C) 1988  */
/*  Copyright (C) 1989  */
/*  Copyright (C) 1992  */
/*  Copyright (C) 1994  */
/*  Harold V. McIntosh  */

# define stdaux stderr

# include <math.h>
# include <stdio.h>
# include "rec.h"
// # include "rectbl.h"
# include "nxlcau2t.h"
# include "keybrd.h"

extern char   arr1[AL];
extern char   ttrule[TS];
extern char   srule[2*KK-1][TS-2];
extern char   cstr[CLEN];			/* console rec program */
extern char   recrule[][48];
extern char   drectabl[][HCH];
extern char   ascrule[KK][KK][KK][KK];
extern int    binrule[KK][KK][KK][KK];
extern int    rij, rdi, rdj;			/* rec demo pointers */
extern int    co, dn;				/* connectivity, dimension */
extern int    cp;				/* critical point          */
extern int    pr, pl;				/* rule, line prob, mils   */
extern int    bi;				/* bernstein index	   */
extern int    ct0, ct1, ct2, ct3;		/* general counters	   */
extern int    sw0, sw1, sw2, sw3;		/* general switches	   */
extern int    ex, wy;				/* hiplot coordinates	   */
extern double blpr[KK];				/* probability from blocks */
extern double nu0, nu1, nu2, nu3;		/* general numeric values  */
extern double mf;				/* mouse increment	   */
extern double sm[TS];				/* selfconsistent nbrhd    */
extern double sn[TS];				/* selfconsistent nbrhd    */
extern double bx[BD];				/* bernstein factor        */
extern double bp[BD];
extern double pv[10];				/* polynomial vector 	   */
extern double fpa, fpb, fpc;			/* fixed points		   */

extern Inst   rprg[];				/* rec program array */
extern struct fptbl dtbl[];

extern long int r_intpar;
extern double   r_dblpar;


# define GROW	   12	/* row for graph panels		*/
# define BROW	    3	/* row for bar charts		*/
# define EROW	    1	/* row for evolution synopsis	*/

# include "nxprob2r.c"
# include "nxprobkt.c"


extern int rec_c(char *z, Inst rprg, int l, struct fptbl dtbl[]);
int rec(char *z);

/* edit the probability screen	*/
edtri() {char c; int i;

  videomode(COLGRAF);
  videopalette(YELREGR);
  prmenu();

  while (0<1) {
  runu();
  videocursor(0,0,36); videoputc('?',2);
  c=kbdin();
  if (c == '\015') break;
  videocursor(0,0,38); videoputc(c,2);
  videocursor(0,0,36); videoputc(' ',0);
  switch (c) {

    case Kf4: if (rec_c(cstr,rprg,PLEN,dtbl)==1) rec_x(rprg); break;
    case Kf5: if (rec_c(&recrule[rij],rprg,PLEN,dtbl)==1) rec_x(rprg); break;

    case KF4: clep('e'); videocursor(0,EROW,0); printf("%0.40s",cstr); break;
    case KF5: clep('e'); videocursor(0,EROW,0); printf("%0.40s",&recrule[rij]); break;



/*  case INS: twodi(dij); break;			/* INSERT key */
/*
/*  case DEL: videoclear(); break;		/* DELETE key */
/*
/*  case PGU: dij=(DROWS+dij-1)%DROWS; twodi(dij); break;/* PAGE UP key */
/*
/*  case PGD: dij=(DROWS+dij+1)%DROWS; twodi(dij); break;/* PAGE DOWN key */
/*
/*  case KGU: dij=1; twodi(dij); break; 		/* ^PAGE UP key */
/*
/*  case KGD: dij=DROWS; twodi(dij); break;		/* ^PAGE DOWN key */

    case PPU: rij=(RROWS+rij-1)%RROWS; recdi(rij); break;/* PAR UP key	 */
    case PPD: rij=(RROWS+rij+1)%RROWS; recdi(rij); break;/* PAR DOWN key  */
    case KPU: rij=0; recdi(rij); break;			/* ^PAR UP key	 */
    case KPD: rij=RROWS-1; recdi(rij); break;		/* ^PAR DOWN key */

    case '+': videopalette(WHCYMAG); break;
    case '-': videopalette(YELREGR); break;
    case 'a': asfreq(3); break;
    case 'A': psevolve(); break;
    case 'b': rprob(pl); break;
    case 'B': pl=numin(0); prmenu(); break;
    case 'c': traj(numin(0),8,3); break;
    case 'C': trajy(numin(0),0); break;
    case 'd': trajy(numin(0),2500); break;
    case 'D': trajy(numin(0),50000); break;
    case 'e': pevolve(); break;
    case 'E': plevolve(); break;
    case 'f': ranchay(ascrule,pr,DS); break;
    case 'F': pr=numin(0); prmenu(); break;
    case 'g': lifreq(50,1,2); break;
    case 'G': lifreq(200,1,1); break;
    case '.': lifreq(1,1,2); break;
    case 'h': egraf(0,100); break;
    case 'i': piter(100,2,3); break;
    case 'j': piter(100,3,3); break;
    case 'J': piter(100,numin(0),3); break;
    case 'k': piter(100,4,3); break;
    case 'l': piter(100,5,3); break;
    case 'L': lesq(50,2); break;
    case 'H': lesq(3,2); break;
    case 'm': moncar(1,2); break;
    case 'M': moncar(50,1); break;
    case 'n': clep('m'); neisum(AL,25); break;
    case 'N': clep('m'); nbrsum(AL,10); break;
    case 'o': co=numin(0); prmenu(); break;
    case 'O': dn=numin(0); prmenu(); break;
    case 'p': ptseq(100); break;
    case 'P': hipro(100,0); break;
    case 'r': ranmf(100,numin(0),0,2); break;
    case 'R': ranmf(200,numin(0),1,2); break;
    case 's': lifreq(50,3,2); break;
    case 'S': lifreq(50,2,2); break;
    case 'X': bx[bi]=smq(); prmenu(); break;
    case 'I': bi=numin(0); prmenu(); break;
    case 'x': pdiff(100); break;
    case 'y': pdiff3(100); break;
    case 'w': pdiff4(100); break;
    case 'v': inlay(4,4,3); inlay(6,2,8); break;
    case 'V': cubch(12); break;
    case 't': twoblockfreq(100); break;
    case '1': clep('m'); oneblfreq(300,48); break;
    case '2': clep('m'); twoblfreq(300,48); break;
    case '3': clep('m'); thrblfreq(300,48); break;
    case '4': clep('m'); foublfreq(300,48); break;
    case '5': clep('m'); fivblfreq(300,48); break;
    case '6': clep('m'); sixblfreq(300,48); break;
    case '7': clep('m'); svnblfreq(300,48); break;
    case '?': trmenu(); break;
    case '!': prmenu(); break;
    case 'z': clep(kbdin()); break;
    case '/': videomode(COLGRAF); videopalette(YELREGR); break;
    case '*': bcprin(); break;
    case '#': parfix(); break;
    case '&': sctf(100,smq()); break;
    case '$': for (i=0; i<11; i++) sctf(100,0.0999*((double)i)); break;
    case '%': clep('m'); neidif(8*BROW); break;
    case 'K': neicon(); break;

    case '=': aijpanel(ct1/3,ct1%3,ct3++,3); break;
    case '|': while (!kbdst()) {onegen(AL); scanaij(1,1,1);} kbdin(); break;
    case '~': loclhden(3); break;

    case '@': sw0=numin(0); break;
    case '^': hiplot00(); break;
    case 'Z': hiplotxx(); break;

    case MOD: gbmar(bi,1); if (bi>0) bi--; gbmar(bi,2); prmenu(); break;
    case MOU: gbmar(bi,1); if (bi<TS-1) bi++; gbmar(bi,2); prmenu(); break;
    case MOR: bx[bi]+=mf; prmenu(); break;
    case MOL: bx[bi]-=mf; prmenu(); break;
    case MOA: mf*=0.5; prmenu(); break;
    case MOB: mf*=2.0; prmenu(); break;
    case MOX: for (i=0; i<BD; i++) bx[i]=1.0; mf=0.05; prmenu(); break;

    default: break;
    }  /* end switch */
  }    /* end while  */
  videopalette(WHCYMAG);
  videomode(T80X25);
}      /* end edtri  */

/* ----------------------------------- */
/* rec variants of the menu operations */
/* ----------------------------------- */
/* -------------------------------------------*/
/* definition of REC operators and predicates */
/* -------------------------------------------*/

// rec(z) char *z; {if (rec_c(z,rprg,PLEN,dtbl)==1) rec_x(rprg);}

extern int	rec_x(Inst rprg);

int kbint() {if (kbdst()) {kbdin(); return 0;} else return 1;}
int dpred() {if (rand()%1000>100) return 0; else return 1;}
int rpred() {if (rand()%1000>499) return 0; else return 1;}

int tilde() {prmenu();}

ropla() {asfreq(3);}
roplb() {rprob(pl);}
ropld() {diagl(*r_pc++);}
rople() {loclhden(1+ct2%3);}
roplf() {frame(*r_pc++);}
roplg() {retgen(2);}
ropli() {piter(100,nibbl(*r_pc++),3);}
ropln() {clep('m'); neisum(AL,1);}
roplo() {co=nibbl(*r_pc++);}
roplp() {videodot((int)(nu1),ct2,1+ct2%3); ct2=(100+ct2+1)%100;}
roplm() {ranmf(100,*r_pc++,0,2);}
roplt() {twoblockfreq(100);}
roplx() {engen(*r_pc++);}
roplc() {clep(*r_pc++);}

ropua() {psstrip(ct0); ct0=(ct0+1)%3;}
ropub() {pl=nibbl(*r_pc++);}
ropud() {scanaij(ct1/3,ct1%3,1+ct2%3); ct1=(ct1+1)%9;}
ropue() {aijpanel(ct1/3,ct1%3,3,1+ct2%3); ct1=(ct1+1)%9;}
ropug() {retgen(1+ct2%3);}
ropuk() {neicon();}
ropuo() {dn=nibbl(*r_pc++);}
ropuq() {cubch(8);}
ropur() {ranmf(100,*r_pc++,1,2);}
roput() {nutoto((int)r_dblpar); totoasc(); runu();}


/* identify the current REC program (in panel E) */
recdi(n) int n; {
clep('e');
videocursor(0,EROW,0); printf(" f4 executes ed. program; f5, demo"); 
videocursor(0,EROW+1,0); printf("%0.40s",&drectabl[n]);
}

/* ----------------------------------------------------------------- */

/* calculate and display local densities */

loclhden(l) int l; {int i0, i1, i2, i, j, k, s, t[2][2][1];
  for (i0=0; i0<2; i0++)
  for (i1=0; i1<2; i1++)
  for (i2=0; i2<1; i2++) t[i0][i1][i2]=0;

  for (i0=0; i0<32; i0+=16)
  for (i1=0; i1<32; i1+=16)
  for (i2=0; i2<16; i2+=16) {
    s=0;
    for (i=0; i<16; i++)
    for (j=0; j<16; j++)
    for (k=0; k<16; k++)
      s+=arr1[ix3(i0+i,i1+j,i2+k)];
    t[i0/16][i1/16][i2/16]+=s;
    }

  for (i0=0; i0<2; i0++)
  for (i1=0; i1<2; i1++)
  for (i2=0; i2<1; i2++) {
    s=i0+2*(i1+2*i2);
    videodot(t[i0][i1][i2]/40,5*s,l);
    }
}

/* calculate and display local densities */

locl8den(l) int l; {int i0, i1, i2, i, j, k, s, t[4][4][2];
  for (i0=0; i0<4; i0++)
  for (i1=0; i1<4; i1++)
  for (i2=0; i2<2; i2++) t[i0][i1][i2]=0;

  for (i0=0; i0<32; i0+=8)
  for (i1=0; i1<32; i1+=8)
  for (i2=0; i2<16; i2+=8) {
    s=0;
    for (i=0; i<8; i++)
    for (j=0; j<8; j++)
    for (k=0; k<8; k++)
      s+=arr1[ix3(i0+i,i1+j,i2+k)];
    t[i0/8][i1/8][i2/8]+=s;
    }

  for (i0=0; i0<4; i0++)
  for (i1=0; i1<4; i1++)
  for (i2=0; i2<2; i2++) {
    s=i0+4*(i1+4*i2);
    videodot(t[i0][i1][i2]/5,3*s,l);
    }
}

/* calculate and display local densities */

locl4den(l) int l; {int i0, i1, i2, i, j, k, s, t[8][8][4];
  for (i0=0; i0<8; i0++)
  for (i1=0; i1<8; i1++)
  for (i2=0; i2<4; i2++) t[i0][i1][i2]=0;

  for (i0=0; i0<32; i0+=4)
  for (i1=0; i1<32; i1+=4)
  for (i2=0; i2<16; i2+=4) {
    s=0;
    for (i=0; i<4; i++)
    for (j=0; j<4; j++)
    for (k=0; k<4; k++)
      s+=arr1[ix3(i0+i,i1+j,i2+k)];
    t[i0/4][i1/4][i2/4]+=s;
    }

  for (i0=0; i0<8; i0++)
  for (i1=0; i1<8; i1++)
  for (i2=0; i2<4; i2++) {
    s=i0+8*(i1+8*i2);
    videodot((int)((100.0*(double)t[i0][i1][i2])/64.0),s%100,l);
    }
}

/* ----------------------------------------------------------------- */

/* initialize some layers for the 3-d automaton */

inlay(i,j,k) int i, j, k; {int i0, i1;
  for (i0=0; i0<25; i0++)
  for (i1=0; i1<25; i1++) {
    arr1[ix3(i+i0,j+i1,k-1)]=0;
    arr1[ix3(i+i0,j+i1,k)]=1;
    arr1[ix3(i+i0,j+i1,k+1)]=0;
    }
}

/* make a cubical checkerboard */
cubch(n) int n; {int i0, i1, i2;
if (n==0) return;
  for (i0=0; i0<32; i0++)
  for (i1=0; i1<32; i1++)
  for (i2=0; i2<16; i2++)
    arr1[ix3(i0,i1,i2)]=(i0/n+i1/n+i2/n)%2;
}

/* -------------------------------------- */
/* show the evolution of a planar section */
/* -------------------------------------- */

plevolve() {int i, j, l, i0, i1, i2;
clep('a');
gfram(100,3);
asctobin();
for (i=0; i<3; i++) for (j=0; j<3; j++) 
  {aijpanel(i,j,3,(i+j)%2+1); onegen(AL);}
}

/* show single ij subpanel of panel A */

aijpanel(i,j,k,l) int i, j, k, l; {int i0, i1, i2, ll;
  switch (dn) {
    case 2: case 5: case 7:
      for (i0=0; i0<32; i0++)
      for (i1=0; i1<32; i1++) {
        ll=arr1[ix2(i0,i1)];
        videodot(i1-33*j,i0+33*i,ll==0?0:l);
        }
      break;
    case 3: default:
      i2=k;
      for (i0=0; i0<32; i0++)
      for (i1=0; i1<32; i1++) {
        ll=arr1[ix3(i0,i1,i2)];
        videodot(i1-33*j,i0+33*i,ll==0?0:l);
        }
      break;}
}

/* ------------------------------------------- */
/* show the evolution of three planar sections */
/* ------------------------------------------- */

psevolve() {int i, l, i0, i1, i2;
  clep('a');
  gfram(100,3);
  asctobin();
  for (i=0; i<3; i++) {
    aijpanel(i,0,3,i%2+1);
    aijpanel(i,1,4,i%2+2);
    aijpanel(i,2,5,i%2+1);
    }
  onegen(AL);
}

/* show a single strip of psevolve for use in a rec program */

psstrip(i) int i; {
aijpanel(i,0,3,i%2+1); aijpanel(i,1,4,i%2+2); aijpanel(i,2,5,i%2+1);}

/* run through all 16 slices of 32x32x16 cube */
scanaij(i,j,l) int i, j, l; {int k; for (k=0; k<16; k++) aijpanel(i,j,k,l);}

/* evaluate the nth generation Bernstein polynomial at point p */

double bern(p,n) double p; int n; {
int    i, j, d;
double q, s, pq[BD];
d=3*n-2; if (dn==2) d=n*n;
pq[0]=q=1.0-p;
for (i=1; i<=d; i++) pq[i]=p;
for (i=1; i<d; i++) {
  for (j=0; j<=i; j++) pq[j]*=q;
  for (j=i+1; j<=d; j++) pq[j]*=p;
  }
for (i=0, s=0.0; i<=d; i++) s+=bp[i]*pq[i];
return(s);
}

/* evaluate the parameters for two-block probabilities after one generation */

twoblock(z) double *z; {
int    i, i0, i1, i2, i3, i4;
int    j, j0, j1;
double w, x, y, p, q, pp;
double a[KK][KK], b[KK], c[KK][KK];

p=z[0]; q=1.0-p; pp=z[1];
a[1][1]=pp; a[1][0]=a[0][1]=p-pp; a[0][0]=1.0-2*p+pp;
b[1]=p; b[0]=q;
for (i=0; i<KK; i++) {for (j=0; j<KK; j++) c[i][j]=0.0;}
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  j0=ascrule[i0][i1][i2][i3]-'0';
  j1=ascrule[i1][i2][i3][i4]-'0';
  x=a[i0][i1]*a[i1][i2]*a[i2][i3]*a[i3][i4];
  y=b[i1]*b[i2]*b[i3];
  w=0.0; if (y>0.0001) w=x/y;
  c[j0][j1]+=w;
  }
z[0]=c[1][0]+c[1][1]; z[1]=c[1][1];
}

/* graph the 2-block parameter differences to find self-consistent values */
/* nxn points 								  */

twoblockfreq(n) int n; {
int    i, j, l;
double op[2], np[2], s, t, u, v;

gfram(100,3);
printf("hello from twoblockfreq\n");
s=1.0/((double)(n));
for (op[1]=s, i=1; i<n; op[1]+=s, i++) {
  for (op[0]=s, j=1; j<n; op[0]+=s, j++) {
    np[0]=op[0]; np[1]=op[1]; twoblock(np);
    u=np[0]-op[0]; v=np[1]-op[1];
    t=u*u+v*v;
    if (t<0.0025) l=0; else {
    if (t<0.0675) {if((i+j)%2) l=0; else l=3;} else {
    if (t<0.3906) l=3; else {
    if (t<0.7500) {if((i+j)%2) l=3; else l=2;} else {
    if (t<1.5000) l=2; else {
    if (t<3.0000) {if((i+j)%2) l=2; else l=1;} else {
    if (t<6.0000) l=1; else {
                  if((i+j)%2) l=1; else l=0;
       }}}}}}}
    videodot(i,j,l);
    }  /* end for-j */
  }    /* end for-i */
}

/* --------------------------------------- */
/* some programs that belong mostly to rec */
/* --------------------------------------- */

/* show the rule number in panel R */

runu() {
  videocursor(0,0,0);  videoputc(':',2);
  if (istot()) {scrstr("  T: "); tscrrul(); scrstr("     LCAU2T        ");}
    else if (isstot()) {scrstr(" S:");
	sscrrul(0); scrstr(",");
	sscrrul(1); scrstr(",");
	sscrrul(2); scrstr(" LCAU2T");}
    else scrrul();
}

/* show a frame */
frame(c) char c; {switch (c) {
  case 'a': gfram(100,3); break;
  case 'b': gfram(100,3); break;
  case 'c': gfram(100,3); break;
  case 'H': hiplot(20,80,0); hiplot(20,180,1); hiplot(120,180,1);
		hiplot(120,80,1); hiplot(20,80,1); break;
  default: break;}}

/* show a diagonal */
diagl(c) char c; {switch (c) {
  case 'a': gdiag(100,3); break;
  case 'b': gdiag(100,3); break;
  case 'c': gdiag(100,3); break;
  case 'H': hiplot(20,80,0); hiplot(120,180,1); hiplot(20,80,0); break;
  default: break;}}

/* graph mean field for nth generation */
engen(c) char c; {switch (c) {
  case '2': pdiff(100);  if (sw0>0) hipbgraf(2); break;
  case '3': pdiff3(100); if (sw0>0) hipbgraf(3);  break;
  case '4': pdiff4(100); if (sw0>0) hipbgraf(4);  break;
  default: break;}}

/* display state frequencies in arr1 as a point on probability graph */
retgen(l) int l; {int j, os, ns; double of, nf, s;
s=100.0/((double)AL);
asctobin();
for (j=0, os=0; j<AL; j++) if (arr1[j]==1) os++;
onegen(AL);
for (j=0, ns=0; j<AL; j++) if (arr1[j]==1) ns++;
nu0=of=s*((double)os); nu1=nf=s*((double)ns);
videodot((int)(of),(int)(nf),l);
}

/* scan the mean field curve using random cell densities */
ranmf(n,g,h,l) int n, g, h, l; {
int     i, j, pp, dp, os, ns;
double of, nf, s;
dp=1000/n;
s=100.0/((double)AL);
asctobin();
for (pp=0, i=0; i<n; pp+=dp, i++) {
  rprob(pp);
  if (g>0) for (j=0; j<g; j++) onegen(AL);
  for (j=0, os=0; j<AL; j++) if (arr1[j]==1) os++;
  onegen(AL);
  for (j=0, ns=0; j<AL; j++) if (arr1[j]==1) ns++;
  of=s*((double)os); nf=s*((double)ns);
  if (h==0) {videodot((int)(0.1*pp),(int)(nf),l);
             if (sw0==1) hiplot(20+(int)(0.1*pp),80+(int)nf,(i==0)?0:1);}
  if (h==1) {videodot((int)(of),(int)(nf),l);
             if (sw0==1) hiplot(20+(int)of,80+(int)nf,(i==0)?0:1);}
  }
}

/* graph Bernstein polynomial on the hiplot plotter */

hipbgraf(k) int k; {int i; double bern(), p;
for (i=0,p=0.0; i<100; i++,p+=0.01) {hiplot(20+i,80+(int)(100.0*bern(p,k)),1);}}

/* ------------------ */
/* hiplot subroutines */
/* ------------------ */



/* show the rule number in panel at top of page */

hirunu() {int i;
  hiplot(10,186,0);
  if (istot()) {fprintf(stdaux,"S12+T:");
     for (i=0; i<TS; i++) fprintf(stdaux,"%c",ttrule[i]);
     fprintf(stdaux,"          LCAU2T");
     fprintf(stdaux,"_");}
    else if (isstot()) {fprintf(stdaux,"S12+S:");
     for (i=0; i<TS-2; i++) fprintf(stdaux,"%c",srule[0][i]);
     fprintf(stdaux,", ");
     for (i=0; i<TS-2; i++) fprintf(stdaux,"%c",srule[1][i]);
     fprintf(stdaux,", ");
     for (i=0; i<TS-2; i++) fprintf(stdaux,"%c",srule[2][i]);
     fprintf(stdaux,"._");}
    else {};
}

hidate() {long cl[1]; char *ctime(); time(cl);
  hiplot(60,10,0); fprintf(stdaux,"S11+LCAU2T_");
  hiplot(60,10,0); fprintf(stdaux,"S11+LCAU2T_");
  hiplot(60,10,0); fprintf(stdaux,"S11+LCAU2T  -  %26s _",ctime(cl));}

hiplot(x,y,l) int x, y, l; {switch (l) {
case 0: fprintf(stdaux,"U %4d,%4d ",10*x,10*y); break;
case 1: fprintf(stdaux,"D %4d,%4d ",10*x,10*y); break;
default: break;}}

hifplot(x,y,l) int x, y, l; {switch (l) {
case 0: fprintf(stdaux,"U %4d,%4d ",x,y); break;
case 1: fprintf(stdaux,"D %4d,%4d ",x,y); break;
default: break;}}

hiplot00() {fprintf(stdaux,";: P1 EC5 V1 A "); hidate(); hirunu();}
hiplotxx() {fprintf(stdaux,"P0 @ ");}



/* end nxprob2t.c */
