
/* QHIL.C					*/
/* Two-Dimensional Half-Integer Square Lattice	*/
/* with binary cells, Moore neighborhood	*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/
/* 12 October 1989 - half-integer radius	*/

# include "qhil.h"
# include "nxlcau2t.h"

# define lcell(i,j) arr1[li+i][lj+j]=1

# define	NX	11	/* number of sample rules */

char xrule[NX][KK][KK][KK][KK];

char qixrule[NX][DS] = {

	"0000000011111111",	/* shift northwest	*/
	"0000111100001111",	/* shift southwest	*/
	"0001011001101001",	/* totalistic rule #20	*/
	"0001011001101000",	/* life candidate 1    	*/
	"0001011001101010",	/* life candidate 2     */
	"0001011101101010",	/* life candidate 3	*/
	"0001010101010111",	/* majority, UL for tie	*/
	"0001011101111111",	/* travelling diamond	*/
	"0011001100110011",	/* shift northeast   	*/
	"0101010101010101",	/* shift southeast   	*/
	"0110100110010110"	/* odd parity          	*/
	};

char    srule[SN][SN];
char    vrule[SN][SN];
char    trule[TN];			/* evolutionary rule */
extern char    ascrule[KK][KK][KK][KK];
extern char    auxrule[KK][KK][KK][KK];
int     aa[NN][NN];				/* loops in edijn */
int     bb[KK][KK][KK][KK];			/* gen 1 de Bruijn table */
int     cc[KK][KK][KK][KK][KK][KK][KK][KK][KK];	/* gen 2 de Bruijn table */
extern int     binrule[KK][KK][KK][KK];
extern int     arr1[AL][AL];			/* planar array */
extern int     arr2[AL][AL];			/* planar array */
int     li, lj;			/* cursor in cell array */
int	lf, ln;			/* live cells, line number */
int     nr[TS];			/* nbrhd representative of symm class */
int	rs[TS];
int     rq[KK][KK][KK][KK];	/* rule use statistics */
int     gc[DS];			/* Golay code key */
int	sc[NC][BD];
int     wc[NC];			/* size (weight) of symm class */
int     tr;			/* trule cursor */

/* copy ASCII rule table to binary rule table */
qasctobin() {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    binrule[i0][i1][i2][i3]=ascrule[i0][i1][i2][i3]-'0';
  }

/* fill ASCII rule table from the semitotalistic rule table */
qsttoasc() {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    ascrule[i0][i1][i2][i3]=srule[i0+i3][i1+i2];
  }

/* fill binary rule table from the totalistic rule table */
totobin() {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    binrule[i0][i1][i2][i3]=trule[i0+i1+i2+i3]-'0';
  }

/* fill ASCII rule table from the totalistic rule table */
qtotoasc() {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    ascrule[i0][i1][i2][i3]=trule[i0+i1+i2+i3];
  }

/* fill trule from ascrule */
asctoto() {
  trule[0]=ascrule[0][0][0][0];
  trule[1]=ascrule[1][0][0][0];
  trule[2]=ascrule[1][1][0][0];
  trule[3]=ascrule[1][1][1][0];
  trule[4]=ascrule[1][1][1][1];
}

/* fill auxrule with blanks */
qauxblnk() {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    auxrule[i0][i1][i2][i3]=' ';
  }

/* fill character array with ASCII zeroes */
arrzer(z) char z[KK][KK][KK][KK]; {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    z[i0][i1][i2][i3]='0';
  }

/* fill integer array with zeroes */
arzerr(z) int z[KK][KK][KK][KK]; {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    z[i0][i1][i2][i3]=0;
  }

/* copy saved rule #n into active rule */
qxtoasc(n) int n; { int i0, i1, i2, i3;
//  xqmenu(n+1);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]=xrule[n][i0][i1][i2][i3];
  }

/* test whether a rule is totalistic */
int qistot() {int i0, i1, i2, i3, l, x[TN];
    l=1;
    x[0]=ascrule[0][0][0][0];
    x[1]=ascrule[1][0][0][0];
    x[2]=ascrule[1][1][0][0];
    x[3]=ascrule[1][1][1][0];
    x[4]=ascrule[1][1][1][1];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
    if (ascrule[i0][i1][i2][i3]!=x[i0+i1+i2+i3]) l=0;      
    return l; }

/* calculate one generation of evolution */
qonegen(arr1,arr2) int  arr1[AL][AL], arr2[AL][AL]; {
int i0, i1, a, b, c, d;

qasctobin();
arzerr(rq);

qframe(1); /* phase to count neighbors */

/* count neighbors of main square */

for (i0=0; i0<AL-1; i0++) {
for (i1=0; i1<AL-1; i1++) {
  a=arr1[i0][i1];
  b=arr1[i0][i1+1];
  c=arr1[i0+1][i1];
  d=arr1[i0+1][i1+1];
  arr2[i0][i1]=binrule[a][b][c][d];
  rq[a][b][c][d]++;
  }}

/* count neighbors for bottom edge (i1=AL-1) */

for (i0=0; i0<AL-1; i0++) {
  a=arr1[i0][AL-1];
  b=arr1[i0][0];
  c=arr1[i0+1][AL-1];
  d=arr1[i0+1][0];
  arr2[i0][AL-1]=binrule[a][b][c][d];
  rq[a][b][c][d]++;
  }

/* count neighbors for right edge (i0=AL-1) */

for (i1=0; i1<AL-1; i1++) {
  a=arr1[AL-1][i1];
  b=arr1[AL-1][i1+1];
  c=arr1[0][i1];
  d=arr1[0][i1+1];
  arr2[AL-1][i1]=binrule[a][b][c][d];
  rq[a][b][c][d]++;
  }

/* count neighbors for bottom right hand corner (i0=AL-1, i1=AL-1) */

  a=arr1[AL-1][AL-1];
  b=arr1[AL-1][0];
  c=arr1[0][AL-1];
  d=arr1[0][0];
  arr2[AL-1][AL-1]=binrule[a][b][c][d];
  rq[a][b][c][d]++;

qframe(2); /* phase to update cells */
lf=0;
for (i0=0; i0<AL; i0++)
for (i1=0; i1<AL; i1++)
  lf+=arr1[i0][i1]=arr2[i0][i1];
qframe(3); /* phase to show results */
}	  /* end subroutine qonegen */
    
/* display a qscreen of evolution */
screen(z) int  z[AL][AL]; {}

/* display a qframe around the cell array */
qframe(l) int l; {}

/* graph incidence of rule usage */
/* beware limited count capacity  */
qurinci(j) int j; {}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
// qurstat() {int i, j, m, n; double s, t;
// for (i=0, s=0.0; i<TS; i++) s+=(double)rs[i];
// if  (s<1.0) {for (i=0; i<TS; i++) rs[i]=0; return;}
// for (i=0; i<TS; i++) {
//   t=500.0*(((double)rs[i])/s);	/* full scale = RHI/500 population */
//   if (t>=32000.0) t=50.0;	/* larger looks negative */
//   n=(int)t;
//   sdot(i,RHI,0);
//   m=RHI; if (n>RHI) {n=RHI; m=RHI+1;}
//   if (n>0) for (j=0; j<n; j++) sdot(i,j,1);
//   for (j=n; j<m; j++) sdot(i,j,2);
//   rs[i]=0;
//   }
// }

/* graph fraction of live cells */
qlstat(y,n) int y, n; {}

/* generate the Golay code table */
sqgolta() {
int i, j, k;
int i0, i1, i2, i3;
k=0;
for (i3=0; i3<KK; i3++) {
for (i2=0; i2<KK; i2++) {
for (i1=0; i1<KK; i1++) {
for (i0=0; i0<KK; i0++) {
i=qubinq(i0,i1,i2,i3);
j=qubinq(i2,i3,i0,i1);
if (j<i) i=j;
j=qubinq(i0,i1,i2,i3);
if (j<i) i=j;
j=qubinq(i0,i1,i2,i3);
if (j<i) i=j;
j=qubinq(i0,i3,i2,i1);
if (j<i) i=j;
j=qubinq(i3,i2,i1,i0);
if (j<i) i=j;
j=qubinq(i3,i2,i1,i0);
if (j<i) i=j;
j=qubinq(i2,i1,i0,i3);
if (j<i) i=j;
j=qubinq(i0,i1,i2,i3);
if (i==j) {
  gc[i]=k;
  sc[k][i0]++; sc[k][i1]++; sc[k][i2]++; sc[k][i3]++;
  wc[k]=1;
  nr[k]=j;
  k++;
} else {
  gc[j]=gc[i]; wc[gc[i]]++;
  }
/*
printf("  %2X, gc[%3d]=%3d, k=%3d, (%d,%d)\n",j,j,i,k,i0+i2+i4+i6,i1+i3+i5+i7);
*/
}}}}
/*
printf("no. cl. = %3d",k);
*/
}

/* calculate binary equivalent */
int qubinq(i0,i1,i2,i3)
int i0, i1, i2, i3; {
return i0+KK*(i1+KK*(i2+KK*i3));}

/* calculate Golay class of a cell */
int qugolcl(i0,i1,arr1) int  i0, i1, arr1[AL][AL]; {int gn;
gn=arr1[i0][i1];
gn=KK*gn+arr1[i0][i1+1];
gn=KK*gn+arr1[i0+1][i1];
gn=KK*gn+arr1[i0+1][i1+1];
gn=gc[gn];
return gn;
}

/* plot self-consistent probability graph */
qubpanel()  {}

/* graph qbernstein polynomial */
qbgraf(n) int n; {}

/* evaluate the qbernstein polynomial for component i at point t[KK] */
double qbern(i,t) int i; double t[KK]; {
int i0, i1, i2, i3; double a, b, c, d;
qasctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++) {
    for (i2=0, c=0.0; i2<KK; i2++) {
      for (i3=0, d=0.0; i3<KK; i3++)
        if (binrule[i0][i1][i2][i3]==i) d+=t[i3];
      c+=d*t[i2];}
    b+=c*t[i1];}
  a+=b*t[i0];}
return a;
}

/* clear the cell array */
zarry(arr1) int  arr1[AL][AL]; {int i0, i1;
for (i0=0; i0<AL; i0++) for (i1=0; i1<AL; i1++) arr1[i0][i1]=0;}

/* display the rule number on the qscreen in glorious technicolor */
cqscrrul() {}

/* display semitotalistic rule number by sum in colors */
csqscrrul() {}

/* display totalistic rule number by sum */
tqscrrul(l) int l; {}

/* display totalistic rule number by sum in colors */
ctqscrrul() {}

/* display rule number as a hexadecimal */
hqscrrul() {int i; long int k;
  videocursor(0,0,0);
  printf("    ");
  for (i=0,  k=0; i<6;  i++) k=2*k+trule[i]-'0';
  printf("%2.2lX",k);
  for (i=6,  k=0; i<38; i++) k=2*k+trule[i]-'0';
  printf("%8.8lX",k);
  for (i=38, k=0; i<70; i++) k=2*k+trule[i]-'0';
  printf("%8.8lX",k);
  for (i=70, k=0; i<TS; i++) k=2*k+trule[i]-'0';
  printf("%8.8lX",k);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of cross section */
wqscrrul() {int i;
  i=trule[101]-'0';
  i=2*i+trule[68]-'0';
  i=2*i+trule[48]-'0';
  i=2*i+trule[10] -'0';
  i=2*i+trule[68]-'0';
  i=2*i+trule[63]-'0';
  i=2*i+trule[10] -'0';
  i=2*i+trule[0] -'0';
  videocursor(0,0,32);
  printf("     ");
  videocursor(0,0,32);
  printf("[%2d]",i);
  videocursor(0,0,0);
  }

/* restore character in edqrule = red/green */
quresch() {}

/* show character in edqrule = yellow cursor */
shoch() {}

/* restore character in edsrule = red/green */
ressch() {}

/* show character in esdrule = yellow cursor */
shosch() {}

/* restore character in etdrule = red/green */
restch() {}

/* show character in etdrule = yellow cursor */
shotch() {}

/* generate random cells throughout arr1 */
ranarr(arr1) int  arr1[AL][AL]; {int i0, i1, i, c;
i=0;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  if (i%8 == 0) c=rand(); i++;
  arr1[i0][i1]=c%KK; c/=KK;
  }}
}

/* generate random cells in the center of z (e.g. arr1) */
rancen(z) int  z[AL][AL]; {int i0, i1, ia, ib, i, c;
i=0;
for (i0=0; i0<AL; i0++) for (i1=0; i1<AL; i1++) z[i0][i1]=0;
ia=AL/3; ib=2*(AL/3);
for (i0=ia; i0<ib; i0++) {
for (i1=ia; i1<ib; i1++) {
  if (i%8 == 0) c=rand(); i++;
  z[i0][i1]=c%KK; c/=KK;
  }}
}

/* generate a random rule */
ranrul() {int i0, i1, i2, i3, i, j;
i=0;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
  if (i%8 == 0) j=rand(); i++;
  ascrule[i0][i1][i2][i3]=j%KK+'0'; j/=KK;
  }}}}
}

/* generate a random protected rule */
ranprul() {int i0, i1, i2, i3, i, j;
i=0;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
  if (i%8 == 0) j=rand(); i++;
  if (auxrule[i0][i1][i2][i3]==' ') {ascrule[i0][i1][i2][i3]='0'+j%KK; j/=KK;}
  }}}}
}

/* generate a random semitotalistic rule */
ransrul() {int i0, i1, i, j;
i=0;
for (i0=0; i0<SN; i0++) for (i1=0; i1<SN; i1++) {
  if (i%8 == 0) j=rand(); i++;
  srule[i0][i1]=j%KK+'0'; j/=KK;
  }
}

/* generate a random totalistic rule */
rantrul() {int i0, i, j;
i=0;
for (i0=0; i0<TN; i0++) {
  if (i%8 == 0) j=rand(); i++;
  trule[i0]=j%KK+'0'; j/=KK;
  }
}


/* adot(i,j,l) - place dot on automaton screen at (i,j) in color l */
adot(i,j,l) int i, j, l; {videodot(j,i,l);}


#include "nxruin2h.c"
#include "nxruin21.c"

/* end qhil.c */
